// -----------------------------------------------------------------------
// Telemetry
// -----------------------------------------------------------------------

import * as browser from "webextension-polyfill";
import { storageLocalGet, storageLocalSet } from "./storage.js";
import { generateUserId, ping } from "./utils/utils.js";
import { getManagementInfo } from "./browser.js";
import { Survey } from "./survey.js";

const STATS_KEY = "ub_stats";

/**
 * Telemetry manager contains a bunch of static stats that are generated once,
 * and several stats that are generated on the fly each time stats are sent
 * to the server.
 */
export class Telemetry {
  constructor(name, version) {
    this.name = name;
    this.version = version;
    this.survey = new Survey();
    const reOS = /(CrOS \w+|Windows NT|Mac OS X|Linux) ([\d._]+)?/;

    let matches = reOS.exec(navigator.userAgent);

    this.operatingSystem = (matches || [])[1] || "Unknown";

    this.operatingSystemVersion = (matches || [])[2] || "Unknown";

    const reBW =
      /(MSIE|Trident|(?!Gecko.+)Firefox|(?!AppleWebKit.+Chrome.+)Safari(?!.+Edge)|(?!AppleWebKit.+)Chrome(?!.+Edge)|(?!AppleWebKit.+Chrome.+Safari.+)Edge|AppleWebKit(?!.+Chrome|.+Safari)|Gecko(?!.+Firefox))[ /]([\d.apre]+)/;

    matches = reBW.exec(navigator.userAgent);

    this.browserName = (matches || [])[1] || "Unknown";

    if (this.browserName === "Firefox") this.browserFlavor = "F"; // Firefox
    else this.browserFlavor = "E"; // Chrome

    this.browserVersion = (matches || [])[2] || "Unknown";

    this.browserLanguage = navigator.language.match(/^[a-z]+/i)[0];

    this.storageStatsAttr = {
      userId: null,
      totalPings: 0,
    };
  }

  async getStatsSettings() {
    if (this.storageStatsAttr.userId != null) {
      return this.storageStatsAttr;
    }
    const data = await storageLocalGet(STATS_KEY);
    const entries = (data && data[STATS_KEY]) || {
      userId: generateUserId(),
      totalPings: 0,
    };
    this.storageStatsAttr = entries;
    return entries;
  }

  /**
   * Get the stats document for the extension. Currently, the stats data only
   * contains "static" information about the extension since we do not track
   * things like block counts.
   *
   * @param userId {string} id of the user
   * @return {{bv: (*|string), u, v: string, f: (string|*), ov: (*|string), l: string, n: string, o: (*|string)}} the current stats
   */
  getStats(userId) {
    return {
      n: this.name,
      v: this.version,
      u: userId,
      f: this.browserFlavor,
      o: this.operatingSystem,
      bv: this.browserVersion,
      ov: this.operatingSystemVersion,
      l: this.browserLanguage,
    };
  }

  async sendStats() {
    const settings = await this.getStatsSettings();
    settings.totalPings = settings.totalPings + 1;
    await storageLocalSet({ [STATS_KEY]: settings });
    return this.sendPing(settings.userId);
  }

  async sendPing(userId) {
    const data = this.getStats(userId);

    if (this.browserName === "Chrome") {
      const info = await getManagementInfo();
      if (info && typeof info.installType === "string") {
        data["it"] = info.installType.charAt(0);
      }
    }
    ping(data)
      .then((response) => {
        if (!response.ok) {
          return;
        }
        return response.json();
      })
      .then((action) => {
        this.survey.handle(action);
      })
      .catch((/*err*/) => {
        // should we log this error?
        // console.log("ping error", err);
      });
  }

  /**
   * Retrieve the next time a ping should occur.
   *
   * @return {Promise<number>} a promise that resolves to the number of minutes between pings
   */
  async getNextScheduleTiming() {
    const details = await this.getStatsSettings();
    let totalPings = details.totalPings;

    if (typeof totalPings !== "number" || isNaN(totalPings)) {
      totalPings = 0;
    }

    let delayHours;

    if (totalPings < 1) {
      // Ping one hour after install
      delayHours = 1;
    } else {
      // Then daily forever
      delayHours = 24;
    }

    return 60 * delayHours;
  }

  async scheduleStatsEvent() {
    const delayTiming = await this.getNextScheduleTiming();
    browser.alarms.create("stats.ping", {
      periodInMinutes: delayTiming,
    });
    browser.alarms.onAlarm.addListener(async () => await this.sendStats());
  }
}
